/* config.c  */
#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include "amc.h"
#include "timer.h"
#include "i2c.h"
#include "drawers.h"

bank1 struct drawer_configuration drawer[MAX_DRAWERS] =
{
  {0,0,0,0},{0,0,0,0},{0,0,0,0},{0,0,0,0},
  {0,0,0,0},{0,0,0,0},{0,0,0,0},{0,0,0,0}
};

/* ------------------- AMC_Config -------------------------------------------

 determines drawer & bin masks automatically when invoked.
 For each drawer shown as present in the drawer mask created
 by the AMC_DrawersOpen function, if it is a smart drawer,
 it is queried for it bin status. The inverse of the bin status
 is taken to indicate the presence of an open bin.

  For example: when a drawer that has no bins is queried,
  it will return FF FF FF. When a drawer that has 24 (all)
  bins, and they are all closed, it will return 00 00 00.
  When the I2C byte is returned from the query, it is inverted,
  so that a drawer with all bins present and closed will be seen
  as returning FF (only for the purposes of this autoconfig). In
  this way, a 1 in the bitmap indicates the presence of a bin,
  and a 0 indicates the absence.

  NOTE: all drawers and bins must be closed when this function
  is invoked, which is at cart boot time.

--------------------------------------------------------------------------- */
void AMC_AutoConfig()
{
BYTE drawer_index;
BYTE inbyte;
BYTE i2cAddress;

    activeDrawerMask = ~AMC_DrawersOpen();
    for (drawer_index = 0, i2cAddress = 0x20; drawer_index < 16; drawer_index++, i2cAddress+=2)
    {
        if (!(activeDrawerMask & (0x01 << drawer_index) ) )
          continue; /* if there is no drawer here, skip it */
        U_I2CStart();
        U_I2CWrite(i2cAddress);
        AMC_BusyLoop(20);
        U_I2CWrite(0xff);
        AMC_BusyLoop(20);

        U_I2CStart();
        U_I2CWrite(i2cAddress+1);
        AMC_BusyLoop(50);

        inbyte = U_I2CRead(I2C_MORE);
        drawer[drawer_index].bin_mask1to8 = ~inbyte;
        AMC_BusyLoop(30);

        inbyte = U_I2CRead(I2C_MORE);
        drawer[drawer_index].bin_mask9to16 = ~inbyte;
        AMC_BusyLoop(30);

	      inbyte = U_I2CRead(I2C_LAST);
        drawer[drawer_index].bin_mask17to24 = ~inbyte;

        U_I2CStop();

        if ( (drawer[drawer_index].bin_mask1to8 == 0x00) &&
             (drawer[drawer_index].bin_mask9to16 == 0x00) &&
             (drawer[drawer_index].bin_mask17to24 == 0x00) )
        {
          drawer[drawer_index].i2c_address = 0x00;
        }
        else
        {
          drawer[drawer_index].i2c_address = i2cAddress;
        }

    } /* end for */
}



/* ------------------- AMC_DisplayConfig ----------------------------------

 displays the current cart configuration:

--------------------------------------------------------------------------- */
void AMC_DisplayConfig()
{
static bank3 int               d;
struct time_stamp dt;
static bank3 BYTE bits[3],c,i=4,j;

  AMC_LoadConfig();
  putch('\n');putch('\r');putch('a');
  mU_SET_LEADING_ZERO(TRUE);
  mU_SET_MAX_WIDTH(2);
  U_PrintDec(cart.scbConnectTimeout);
  U_PrintDec(cart.sleepTimeout);
  U_PrintDec(cart.drawerOpenTimeout);
  putch('\n');putch('\r');putch('b');
  putch(cart.accessCode[0]);putch(cart.accessCode[1]);putch(cart.accessCode[2]);putch(cart.accessCode[3]);
  AMC_ReadClock(&dt);
  putch('\n'); putch('\r'); putch('c');
  mU_SET_MAX_DECIMAL(99);mU_SET_MIN_DECIMAL(1);
  U_PrintDec(dt.year);
  mU_SET_MAX_DECIMAL(12);
  U_PrintDec(dt.month);
  mU_SET_MAX_DECIMAL(31);
  U_PrintDec(dt.day);
  putch('\n'); putch('\r'); putch('d');
  mU_SET_MAX_DECIMAL(23);mU_SET_MIN_DECIMAL(0);
  U_PrintDec(dt.hour);
  mU_SET_MAX_DECIMAL(59);
  U_PrintDec(dt.minute); U_PrintDec(dt.second);
  putch('\n');putch('\r');
  
  for ( d=7;d>=0; d--)
  {
    if  (activeDrawerMask & (0x01 << d))
    {
      if (drawer[d].i2c_address )
        putch ('S');
      else
        putch ('D');
    }
    else
      putch('-');
  }

  for (d = 0; d < MAX_DRAWERS; ++d)
  {
    if (!drawer[d].i2c_address)
      continue;
    putch('\n'); putch('\r');
    U_PrintHex(drawer[d].i2c_address);
    putch('\n'); putch('\r');

    bits[0] = (drawer[d].bin_mask1to8 << 4) + (drawer[d].bin_mask1to8 >> 4);
    bits[1] = (drawer[d].bin_mask9to16 << 4) + (drawer[d].bin_mask9to16 >> 4);
    bits[2] = (drawer[d].bin_mask17to24 << 4) + (drawer[d].bin_mask17to24 >> 4);

    for (i=3;i;--i)
    {
      for (j=0;j<8;j++)
      {
         c = bits[i-1] & (0x01 <<j) ? 'B' : '-';
         putch(c);
         if (j==3)
         {
           putch('\n');
           putch('\r');
         }
      }
      putch('\n');
      putch('\r');
    }
  }
}


/* ------------------- AMC_Config ----------------------------------

 AMC_Config

 gets configuration parameters through serial port and
 writes them to eeprom

------------------------------------------------------------------- */

void AMC_Config()
{
int                 i;
char                instring[BUFFER_SIZE];
struct time_stamp   dt;

   U_Puts("\n\ra?");
   instring[0]=0;
   U_Gets(instring);
	if (instring[0]!=0)
	{
    	cart.drawerOpenTimeout = (instring[4] - 0x30)*10 + (instring[5] - 0x30);
    	cart.sleepTimeout = (instring[2] - 0x30)*10 + (instring[3] - 0x30);
    	cart.scbConnectTimeout = (instring[0] - 0x30)*10 + (instring[1] - 0x30);
	}
   U_Puts("\n\rb?");
   instring[0]=0;
   U_Gets(instring);
	if (instring[0]!=0)
	{
      for ( i=0; i < 4; ++i )
        cart.accessCode[i] = instring[i];
   }
   U_Puts("\n\rc?");
   instring[0]=0;
   AMC_ReadClock(&dt);
   U_Gets(instring);
   if (instring[0]!=0)
	{
    	dt.day = (instring[4] - 0x30)*10 + (instring[5] - 0x30);
      dt.month = (instring[2] - 0x30)*10 + (instring[3] - 0x30);
      dt.year = (instring[0] - 0x30)*10 + (instring[1] - 0x30);
      dt.century = dt.year < 95 ? 20 : 19;
      AMC_WriteClock(&dt);
	}
   U_Puts("\n\rd?");
   instring[0]=0;
   U_Gets(instring);
	if (instring[0]!=0)
	{
    	dt.second = (instring[4] - 0x30)*10 + (instring[5] - 0x30);
    	dt.minute = (instring[2] - 0x30)*10 + (instring[3] - 0x30);
    	dt.hour = (instring[0] - 0x30)*10 + (instring[1] - 0x30);
      AMC_WriteClock(&dt);
	}
   AMC_SaveConfig();
}
