/* i2c.c - Application level i2c functions */

#include <pic.h>
#include <stdio.h>

#define ULIB_ALL_FUNCTIONS
#include "amc.h"
#include "remoteio.h"

#include "i2c.h"
#include "drawers.h"
#include "timer.h"

extern bank1 struct memory_location location;
extern BYTE                   current_drawer;
extern BYTE                   bin;

void AMC_WriteExtEeprom(struct memory_location * address,BYTE count,BYTE * data)
{
BYTE d;
signed int block;

  for (block = (count < 31)?count:31 ;block > 0;block = (count < 31)?count:31)
  {
    d = 0;
    do
    {
        U_I2CStart();
    }
    while (U_I2CWrite(_24C65_EEPROM_WRITE) && ++d < 5);

    U_I2CWrite(address->high);
    U_I2CWrite(address->low);
    for (d = 0; d < block - 1; ++d)
    {
        U_I2CWrite(*(data++));
    }
    U_I2CWrite(*data);
    U_I2CStop();

    MilliPause(20);
    count -= block;
  }
}

void AMC_ReadExtEeprom(struct memory_location* address,BYTE count,BYTE* data)
{
BYTE d;

  d = 0;
  do
  {
    U_I2CStart();
  }
  while (U_I2CWrite(_24C65_EEPROM_WRITE) && ++d < 5);

  U_I2CWrite(address->high);
  U_I2CWrite(address->low);
  d = 0;

  do
  {
    U_I2CStart();
  }
  while  (U_I2CWrite(_24C65_EEPROM_READ) && ++d < 5);

  for (d = 0; d < count - 1; ++d)
  {
    *(data++) = U_I2CRead(I2C_MORE);
  }
  *data = U_I2CRead(I2C_LAST);
  U_I2CStop();
}


//////////////////// checksum_value ///////////////////////
// calculates the sum of all of the bytes of the cart
// structure (modulo 256)
///////////////////////////////////////////////////////////
BYTE AMC_ChecksumValue()
{
BYTE val;

  val = cart.accessCode [0] + cart.accessCode[1] + cart.accessCode[2] + cart.accessCode[3] 
       + cart.sleepTimeout+cart.scbConnectTimeout + cart.drawerOpenTimeout;
  return val;
}

//////////////////// save_config  /////////////////////////
// copies the contents of the cart variable and
// drawer array to eeprom
///////////////////////////////////////////////////////////
void AMC_SaveConfig()
{
  cart.checksum = AMC_ChecksumValue();
  AMC_WriteExtEeprom(&location,sizeof(struct cart_configuration),&cart.accessCode[0]);

}

//////////////////// load_config  /////////////////////////
// copies the contents of the cart variable and
// drawer array from eeprom
///////////////////////////////////////////////////////////
void AMC_LoadConfig()
{
  AMC_ReadExtEeprom(&location,sizeof(struct cart_configuration),(BYTE*)&cart);
  if ( cart.checksum != AMC_ChecksumValue() )
  {
    U_Puts("h:");   
	 mAMC_SETDEFAULTCONFIGURATION();
  }
}

void AMC_WriteClock(struct time_stamp* dt)
{
BYTE clockData;
BYTE remainder;

  U_I2CStart();
  U_I2CWrite(RTC_CLOCK_WRITE_ADDRESS);   // set register pointer to 0x00
  U_I2CWrite(0);

  // sequentially write the registers containing the time
  remainder = dt->second % 10;
  clockData = 0x10 * (dt->second / 10) + remainder;
  U_I2CWrite(clockData);
  remainder = dt->minute % 10;
  clockData = 0x10 * (dt->minute / 10) + remainder;
  U_I2CWrite(clockData);
  remainder = dt->hour % 10;
  clockData = 0x10 * (dt->hour / 10) + remainder;
  U_I2CWrite(clockData);
  U_I2CWrite(0x01);                     // day of the week
  remainder = dt->day % 10;
  clockData = 0x10 * (dt->day / 10) + remainder;
  U_I2CWrite(clockData);
  remainder = dt->month % 10;
  clockData = 0x10 * (dt->month / 10) + remainder;
  U_I2CWrite(clockData);
  remainder = dt->year % 10;
  clockData = 0x10 * (dt->year / 10) + remainder;
  U_I2CWrite(clockData);
  U_I2CStop();
  MilliPause(20);
}

void AMC_ReadClock(struct time_stamp* dt)
{
BYTE clockData = 0;

  U_I2CStart();
  U_I2CWrite(RTC_CLOCK_WRITE_ADDRESS);  // set register pointer to 0x00
  U_I2CWrite(0);

  U_I2CStart();
  U_I2CWrite(RTC_CLOCK_READ_ADDRESS); // send the read command
  clockData=U_I2CRead(I2C_MORE);
  dt->second = ((clockData & 0x70) >> 4) * 10 + (clockData & 0x0f);
  clockData=U_I2CRead(I2C_MORE);
  dt->minute = ((clockData & 0x70) >> 4) * 10 + (clockData & 0x0f);
  clockData=U_I2CRead(I2C_MORE);
  dt->hour = ((clockData & 0x30) >> 4) * 10 + (clockData & 0x0f);
  clockData=U_I2CRead(I2C_MORE);               // ignore the day of the week
  clockData=U_I2CRead(I2C_MORE);
  dt->day = ((clockData & 0x30) >> 4) * 10 + (clockData & 0x0f);
  clockData=U_I2CRead(I2C_MORE);
  dt->month = ((clockData & 0x30) >> 4) * 10 + (clockData & 0x0f);
  clockData=U_I2CRead(I2C_LAST);
  U_I2CStop();

  dt->year = ((clockData & 0xf0) >> 4) * 10 + (clockData & 0x0f);
  if (dt->year < 98)
    dt->century = 20;
  else
    dt->century = 19;
}

void AMC_WriteDrawer(BYTE drawer_num,BYTE data) // data is the bin number of the led to light
{
BYTE drawer_address;

    drawer_address = drawer[drawer_num-1].i2c_address;
    U_I2CStart();
    U_I2CWrite(drawer_address);
    MilliPause(10);
    U_I2CWrite(data);
    MilliPause(10);
    U_I2CStop();
    MilliPause(10);
}

void AMC_ReadDrawer(BYTE drawer_num,struct bin_mask* bin_status)
{
BYTE drawer_address;


    drawer_address = drawer[drawer_num-1].i2c_address;
    bin_status->bin1to8 = 0;
    bin_status->bin9to16 = 0;
    bin_status->bin17to24 = 0;

    U_I2CStart();
    U_I2CWrite(drawer_address);
    MilliPause(20);
    U_I2CWrite(0xff);
    MilliPause(20);

    U_I2CStart();
    U_I2CWrite(drawer_address+1);
    MilliPause(50);

    bin_status->bin1to8 = U_I2CRead(I2C_MORE) & drawer[drawer_num-1].bin_mask1to8;
    MilliPause(30);

    bin_status->bin9to16 = U_I2CRead(I2C_MORE) & drawer[drawer_num-1].bin_mask9to16;
    MilliPause(30);

    bin_status->bin17to24 = U_I2CRead(I2C_LAST) & drawer[drawer_num-1].bin_mask17to24;

    U_I2CStop();

}


bank2 BYTE clockData;


void AMC_InitI2C()
{
   U_I2CStart();
   U_I2CWrite(RTC_CLOCK_WRITE_ADDRESS);
   U_I2CWrite(RTC_CONTROL_REGISTER);         // set register pointer to 0x07 (the control register)
   U_I2CWrite(RTC_ENABLE_4KHz);          /* enable  4 KHz square wave  */
   U_I2CStop();
   MilliPause(20);
   U_I2CStart();
   U_I2CWrite(RTC_CLOCK_WRITE_ADDRESS);
   U_I2CWrite(0x00);         // set reg pointer to 0x00 (to access the clock enable bit)
   U_I2CStop();
   MilliPause(20);
   U_I2CStart();
   U_I2CWrite(RTC_CLOCK_READ_ADDRESS);
   clockData = U_I2CRead(I2C_LAST);
   U_I2CStop();
   MilliPause(20);
   clockData &= RTC_CLOCK_HALT_BIT_MASK;      // need to rewrite values to ensure clock starts
                            /* this also sets bit 7 to 0 to disable the Clock Halt bit */
                            /* and thereby enable the oscillator if the chip was never used before */

   U_I2CStart();
   U_I2CWrite(RTC_CLOCK_WRITE_ADDRESS);
   U_I2CWrite(0x00);
   U_I2CWrite(clockData);
   U_I2CStop();
   MilliPause(20);
   U_I2CStart();
   U_I2CWrite(RTC_CLOCK_WRITE_ADDRESS);
   U_I2CWrite(RTC_CONTROL_REGISTER);
   U_I2CWrite(RTC_ENABLE_4KHz);         /* enable  4 KHz square wave  */
   U_I2CStop();
   MilliPause(20);

   AMC_LoadConfig();

}
