/* i2c.h - application level i2c stuff */

#ifndef AMC_I2C
#define AMC_I2C

#include "pic.h"
#include "\ulib\udefs.h"
#include "\ulib\ulib.h"  /* lew level i2c stuff */

/* routines for a MicroChip 24C65 EEPROM  */
#define _24C65_EEPROM_SIZE   8192
#define _24C65_EEPROM_WRITE  0xa2
#define _24C65_EEPROM_READ   0xa3

/// communicating with real-time clock DS1307 ////

#define RTC_CONTROL_REGISTER    0x07
#define RTC_ENABLE_4KHz         0x11
#define RTC_SECONDS_REGISTER    0x00
#define RTC_CLOCK_HALT_BIT_MASK 0x7F

#define RTC_CLOCK_WRITE_ADDRESS 0xD0
#define RTC_CLOCK_READ_ADDRESS  0xD1

struct memory_location
{
  BYTE  high;
  BYTE  low;
};

struct time_stamp
{
  BYTE  century;
  BYTE  year;
  BYTE  month;
  BYTE  day;
  BYTE  hour;
  BYTE  minute;
  BYTE  second;
};

struct log_entry
{
  struct time_stamp   stamp;
  char                id[5];
  char                pin[5];
};

/* communicating with smart drawers */
struct bin_mask
{
   BYTE bin1to8;
   BYTE bin9to16;
   BYTE bin17to24;
};

#endif /* AMC_I2C */

/* prototypes */


void AMC_WriteExtEeprom(struct memory_location * address,BYTE count,BYTE * data);
void AMC_ReadExtEeprom(struct memory_location* address,BYTE count,BYTE* data);
void AMC_WriteEepromByte(struct memory_location * address,BYTE data);
BYTE AMC_ReadEepromByte(struct memory_location* address);
BYTE AMC_ChecksumValue();
void AMC_SaveConfig();
void AMC_LoadConfig(void);
void AMC_WriteClock(struct time_stamp* dt);
void AMC_ReadClock(struct time_stamp* dt);
void AMC_WriteDrawer(BYTE drawer_num,BYTE data);
void AMC_ReadDrawer(BYTE drawer_num,struct bin_mask* bin_status);
void AMC_InitI2C(void);


