/* ulib.c */
/* By Kenneth C. Finney.  Copyright 1998,1999. All Rights reserved */
/* Rev. 2.201 (P) */

#include	<stdio.h>
#include	<stdlib.h>
#include	<string.h>

#include <pic.h>

#ifndef XTL
#error XTL (clock frequency) needs to be defined
#endif


#include "\ulib\ulib.h"

#ifdef ULIB_I2C_FUNCTIONS
/* I2C library */
BYTE i2cError=0;

void
U_I2CStart(void)
{
   I2C_CLOCK_LOW;                  /* ensure clock is low */
   MicroPause(5);
	 I2C_DATA_HIGH;                 /* ensure data is high */
   MicroPause(5);
	 I2C_CLOCK_HIGH;                 /* clock pulse high */
   MicroPause(5);
   I2C_DATA_LOW;                 /* the high->low transition */
   MicroPause(5);
   I2C_CLOCK_LOW;                  /* send clock low again */

  return;
}

void
U_I2CStop(void)
{

  I2C_CLOCK_LOW;                  /* ensure clock is low */
  MicroPause(5);
  I2C_DATA_LOW;						       /* data starts out low */
  MicroPause(5);
  I2C_CLOCK_HIGH;                 /* clock needs to be high */
  MicroPause(5);
  I2C_DATA_HIGH;                 /* low-high transition */
  MicroPause(5);

  return;
}


signed char
U_I2CWrite(unsigned char data)
{
char i=8;
  unsigned char ack;

  i2cError=I2C_NO_ERROR;
  while(i--)
  {
    I2C_CLOCK_LOW;
    MicroPause(2);
		SDA =  ((data >> i)  & 0x01);	/* bit to send */
		SDA_DIR = I2C_OUTPUT;
    SCL_DIR=I2C_INPUT;
    MicroPause(6);
  }
  I2C_CLOCK_LOW;
  I2C_DATA_HIGH;      /* disable data line - listen for ack */

  MicroPause(6);      /* SCL low to data out valid */
  I2C_CLOCK_HIGH;       /* float clock high */

  MicroPause(4);

  ack = SDA;        /* read the acknowledge */

/* wait for slave to release clock line after processing byte */

  if (SCL==LOW)
  {
    MicroPause(I2C_TM_SCL_TMO);
    if (SCL==LOW)
    {
      i2cError=I2C_CLOCK_NOT_RELEASED;
      return (I2C_ERROR);
    }
  }
  I2C_CLOCK_LOW;      /* keep control of clock - hold it low */

  return ack;
}


char
U_I2CRead(unsigned char ack)
{
unsigned char byt = 0;
unsigned char i;

  i2cError=I2C_NO_ERROR;

  for(i=0; i<8; i++)
  {
    I2C_CLOCK_LOW;               /* float clock low */
    MicroPause(I2C_TM_SCL_LOW);  /* min clock low  period */
    I2C_DATA_HIGH;               /* release data line */
    I2C_CLOCK_HIGH;              /* float clock high */

    /* wait for slave to release clock line after processing byte */
    if (SCL==LOW)
    {
      MicroPause(I2C_TM_SCL_TMO);
      if (SCL==LOW)
      {
        i2cError=I2C_CLOCK_NOT_RELEASED;
        return (I2C_ERROR);
       }
    }
    MicroPause(I2C_TM_SCL_HIGH);
    byt = byt << 1;                 /* read the next bit */
    byt |= SDA;
  }

  I2C_CLOCK_LOW;                      /* float clock low */
  if (ack==I2C_MORE)
    SDA = LOW;         /* drive line low -> more to come */
  else
    SDA = HIGH;     /* drive line high -> no more wanted */

  SDA_DIR = I2C_OUTPUT;
  MicroPause(I2C_TM_DATA_SU);
  I2C_CLOCK_HIGH;      /* float clock high */

    /* wait for slave to release clock line after processing byte */
  if (SCL==LOW)
  {
    MicroPause(I2C_TM_SCL_TMO);
    if (SCL==LOW)
    {
      i2cError=I2C_CLOCK_NOT_RELEASED;
      return (I2C_ERROR);
     }
  }
  MicroPause(I2C_TM_SCL_HIGH);                  /* wait a bit */
  I2C_CLOCK_LOW;                   /* and then back low again */

  return byt;
}

#endif


#ifdef ULIB_SERIAL_FUNCTIONS

#if defined(_16C73)	|| defined(_16C73A)	||\
    defined(_16C74)	|| defined(_16C74A)	||\
    defined(_16C63)     || defined(_16CR63)     || defined(_16C63A)   ||\
    defined(_16C65B)    || defined(_16C73B)     || defined(_16C74B)   ||\
    defined(_16F873)	|| defined(_16F874)	||\
    defined(_16F876)	|| defined(_16F877) || defined (_16C77) || defined (_16C76)

/* serial library */

/* Routines for initialisation and use of the Serial Port
 * for the PIC 16C... processors.
 */
unsigned char
U_SerialInit(unsigned long int baud)
{
	int X;

	/* calculate and set baud rate register */
	/* for asynchronous mode */

	X = (int)(XTL/(16UL * baud)) - 1;
	if((X>255) || (X<0))
	{
		X = (int)(XTL/(64UL * baud)) - 1;
		if((X>255) || (X<0))
		{
			return 1;	/* baud rate not available */
		}
		else
			BRGH = 0;	/* low baud rate */
	}
	else
		BRGH = 1;	/* high baud rate */
	SPBRG = X;	/* set the baud rate */

	SYNC = 0;	/* asynchronous */
	CREN = 1;	/* enable reception */
	SREN = 0;	/* no effect */
	TXIE = 0;	/* disable tx interrupts */
	RCIE = 0;	/* disable rx interrupts */
	TX9  = 0;	/* 8-bit transmission */
	RX9  = 0;	/* 8-bit reception */
	TXEN = 1;	/* enable the transmitter */

	SPEN = 0;	/* disable serial port pins until we need the port enabled */
	return 0;
}

void
putch(char c)
{

//  MilliPause(10,KICK_DOG);
  MilliPause(10);

  while(!TRMT)	/* set when shift register is empty */
    continue;
  TXREG = c;
}

char
getch(void)
{
  while(!RCIF)	/* set when register is not empty */
	    ;
  return RCREG;
}


unsigned char
U_SerialCheckOERR(void)
{
	if(OERR)	/* re-enable after overrun error */
	{
		CREN = 0;
		MicroPause(1); /* just to let the bit 'settle' */
		CREN = 1;
		return 1;
	}

	return 0;
}

void
U_Puts(const char *string)
{
  while (*string)
  {
  	 putch(*string);
	 string++;
  }
  putch('\0');
}

void
U_Gets(char *string)
{
char c;

  while( (c = getch()) != ';')
  {
    if ( (c=='\r') || (c=='\n') )
	   continue;
  	 *string = c;
	 string++;
	}
}

void
U_PrintHex(BYTE theByte)
{
}

void
U_PrintDec(BYTE theBin)
{
  
}


#ifndef  PRINTF_LONG
#define PRINTF_LONG
#endif
/*
U_Printf(const char * f, ...)
{
char		c;
va_list		ap;
unsigned char	sign;
unsigned char	prec;
unsigned int	i;
signed char	width;

	va_start(ap, f);
	while(c	= *f++)
		if(c !=	'%')
			putch(c);
		else
		{
			prec = 255;
			width = 0;
			sign = 0;
loop:
			switch(c = *f++)
			{
			case 0:
				return;
#if 0
			case '*':
				width = va_arg(ap, int);
				goto loop;
#endif

#ifdef PRINTF_LONG
			case 'l':
				sign |= 0x80;
				goto loop;

#endif

			case 'd':
				sign++;
				goto decimal;
			case 'x':
#if 0
			case 'X':
#endif
				prec += 8;
#if 0
			case 'o':
#endif
				prec -= 2;
#if 0
			case 'u':
#endif

decimal:
				{
				unsigned int	j;

#ifdef PRINTF_LONG
					if(sign & 0x80)
						i = va_arg(ap, unsigned int);
					else
#endif
						i = va_arg(ap, int);
					if(sign & 1)
					{
					       if((int)i < 0)
							 {
								i = -i;
								width--;
					       }
							 else
						       sign = 0;
					}
					else
					{
#ifdef PRINTF_LONG
						if(!(sign & 0x80))
							i = (unsigned short)i;
#endif
						sign = 0;
					}
					prec -= 255-10;

					c = (unsigned int)i % prec;
					*(unsigned int *)&i /= prec;
					j = 1;
					while(j <= i)
					{
						j *= prec;
						width--;
					}
					while(--width > 0)
						putch(' ');
					if(sign & 1)
						putch('-');
					while(j /= prec)
					{
						width = (i/j)%prec;
						if(width > 9)
							width += 'A'-'0'-10;
						putch(width+'0');
					}
					if(c > 9)
						c += 'A'-'0'-10;
					putch(c+'0');
				}
				break;

			case 'c':
				while(width > 1)
				{
					putch(' ');
					width--;
				}
				c = va_arg(ap, int);
				putch(c);
				continue;

			case 's':
				{
					const char *	x;

					x = va_arg(ap, const char *);
					c = 0;
					while(x[c])
						c++;
					if(c < prec)
						prec = c;
					while(width > prec) {
						putch(' ');
						width--;
					}
					while(prec--) {
						c = *x++;
						putch(c);
					}
					continue;
				}

#if 0
			case '.':
				if(*f == '*') {
					prec = va_arg(ap, int);
					f++;
				} else {
					prec = *f++ - '0';
					c = *f;
					if(c >= '0' && c <= '9') {
						prec = prec*10 + c - '0';
						f++;
					}
				}
				goto loop;
#endif

			default:
				if(c >= '0' && c <= '9') {
					width = width * 10 + c - '0';
					goto loop;
				}
				putch(c);
				continue;

			}
		}
}

*/
#endif

#endif

#ifdef ULIB_PAUSE_FUNCTIONS

/*-------------------- Pause routines ---------------------- */

void MilliPause(unsigned long cnt)
{
	do
  {
    CLRWDT();
		MicroPause(1000); /* the code chews up 6 us on each loop */

  }
  while(--cnt);
}


/* for use when a delay is needed within an interrupt service routine */
void
ISRMilliPause(unsigned long cnt)
{
	do
  {
    CLRWDT();
		MicroPause(1000); /* the code chews up 6 us on each loop */

  }
  while(--cnt);
}

#endif
